/*
ELFISection.cpp - ELF section reader implementation.
Copyright (C) 2001 Serge Lamikhov-Center <to_serge@users.sourceforge.net>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <cstring>
#include "ELFIOImpl.h"
#include "ELFIOUtils.h"

ELFISection::ELFISection( IELFI* pIELFI, std::ifstream& stream, Elf32_Shdr* pHeader ) :
    m_pIELFI( pIELFI ), m_stream( stream )
{
    m_pIELFI->AddRef();
    std::memset( &m_sh, sizeof( m_sh ), '\0' );
    m_nRefCnt = 1;
    m_sh      = *pHeader;
    m_data    = 0;
}


ELFISection::~ELFISection()
{
    m_pIELFI->Release();
    delete m_data;
}


int
ELFISection::AddRef() const
{
    return ++m_nRefCnt;
}


int
ELFISection::Release() const
{
    int nRet = --m_nRefCnt;
    if ( 0 == m_nRefCnt ) {
        delete this;
    }

    return nRet;
}


std::string
ELFISection::GetName() const
{
    std::string strRet   = "";
    const char* pName    = 0;
    Elf32_Half  shstrndx = m_pIELFI->GetSecStrNdx();

    if ( SHN_UNDEF != shstrndx ) {
        const IELFISection* pStrSec = m_pIELFI->GetSection( shstrndx );
        pName = pStrSec->GetData();
        if ( 0 != pName ) {
            strRet = pName + Convert32Word2Host( m_sh.sh_name, m_pIELFI->GetEncoding() );
        }
        pStrSec->Release();
    }
    
    return strRet;
}


Elf32_Word
ELFISection::GetType() const
{
    return Convert32Word2Host( m_sh.sh_type, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetFlags() const
{
    return Convert32Word2Host( m_sh.sh_flags, m_pIELFI->GetEncoding() );
}


Elf32_Addr
ELFISection::GetAddress() const
{
    return Convert32Addr2Host( m_sh.sh_addr, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetSize() const
{
    return Convert32Word2Host( m_sh.sh_size, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetLink() const
{
    return Convert32Word2Host( m_sh.sh_link, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetInfo() const
{
    return Convert32Word2Host( m_sh.sh_info, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetAddrAlign() const
{
    return Convert32Word2Host( m_sh.sh_addralign, m_pIELFI->GetEncoding() );
}


Elf32_Word
ELFISection::GetEntrySize() const
{
    return Convert32Word2Host( m_sh.sh_entsize, m_pIELFI->GetEncoding() );
}


const char*
ELFISection::GetData() const
{
    if ( 0 == m_data && SHT_NULL != GetType() && SHT_NOBITS != GetType() &&
         0 != GetSize() ) {
        m_stream.seekg( Convert32Off2Host( m_sh.sh_offset,
                                           m_pIELFI->GetEncoding() ) );
        Elf32_Word size = GetSize();
        m_data = new char[size];
        if ( 0 != m_data ) {
            m_stream.read( m_data, size );
        }
    }

    return m_data;
}
